# coding=UTF-8
# ex:ts=4:sw=4:et=on

# Copyright (c) 2013, Mathijs Dumon
# All rights reserved.
# Complete license can be found in the LICENSE file.

"""
    A python implementation of the matrix algorithm developed for the X-ray
        diffraction analysis of disordered lamellar structures
"""

import pkg_resources
from pyxrd.__version import __version__
