# coding=UTF-8
# ex:ts=4:sw=4:et=on

# Copyright (c) 2013, Mathijs Dumon
# All rights reserved.
# Complete license can be found in the LICENSE file.

from .namespace import project_parsers

from .json_project_parser import JSONProjectParser
from .sybilla_project_parser import SybillaProjectParser