# coding=UTF-8
# ex:ts=4:sw=4:et=on

# Copyright (c) 2013, Mathijs Dumon
# All rights reserved.
# Complete license can be found in the LICENSE file.

from pyxrd.generic.controllers import BaseController

class EditInSituBehaviourController(BaseController):
    """
        The controller for the Behaviour model
    """

    pass # end of class