# coding=UTF-8
# ex:ts=4:sw=4:et=on

# Copyright (c) 2013, Mathijs Dumon
# All rights reserved.
# Complete license can be found in the LICENSE file.

SIGNAL_RUN_LAST = None
TYPE_NONE = None
TYPE_PYOBJECT = None

def type_register(self, *args, **kwargs):
    pass