#!/usr/bin/python
# coding=UTF-8
# ex:ts=4:sw=4:et=on

# Copyright (c) 2013, Mathijs Dumon
# All rights reserved.
# Complete license can be found in the LICENSE file.

"""
    This file contains some flags which (partially) control if certain tests
    are run. This way lengthy tests can be skipped in the default set.
"""

SKIP_REFINEMENT_TEST = True
